/*!
 * @ignorance/vue-validator 0.0.43 (https://github.com/yesixuan/@ignorance/vue-validator)
 * API https://github.com/yesixuan/@ignorance/vue-validator/blob/master/doc/api.md
 * Copyright 2017-2019 yesixuan. All Rights Reserved
 * Licensed under MIT (https://github.com/yesixuan/@ignorance/vue-validator/blob/master/LICENSE)
 */

'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

require('core-js/modules/web.dom.iterable');
require('core-js/modules/es6.object.keys');
require('core-js/modules/es6.array.reduce');
require('core-js/modules/es6.array.for-each');
require('core-js/modules/es6.array.some');
require('core-js/modules/es6.object.assign');
require('core-js/modules/es6.regexp.constructor');
require('core-js/modules/es6.regexp.match');
require('core-js/modules/es6.function.bind');
require('core-js/modules/es6.function.name');

function _classCallCheck(instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
}

function _defineProperties(target, props) {
  for (var i = 0; i < props.length; i++) {
    var descriptor = props[i];
    descriptor.enumerable = descriptor.enumerable || false;
    descriptor.configurable = true;
    if ("value" in descriptor) descriptor.writable = true;
    Object.defineProperty(target, descriptor.key, descriptor);
  }
}

function _createClass(Constructor, protoProps, staticProps) {
  if (protoProps) _defineProperties(Constructor.prototype, protoProps);
  if (staticProps) _defineProperties(Constructor, staticProps);
  return Constructor;
}

function _arrayWithoutHoles(arr) {
  if (Array.isArray(arr)) {
    for (var i = 0, arr2 = new Array(arr.length); i < arr.length; i++) {
      arr2[i] = arr[i];
    }

    return arr2;
  }
}

function _iterableToArray(iter) {
  if (Symbol.iterator in Object(iter) || Object.prototype.toString.call(iter) === "[object Arguments]") return Array.from(iter);
}

function _nonIterableSpread() {
  throw new TypeError("Invalid attempt to spread non-iterable instance");
}

function _toConsumableArray(arr) {
  return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _nonIterableSpread();
}

function _arrayWithHoles(arr) {
  if (Array.isArray(arr)) return arr;
}

function _iterableToArrayLimit(arr, i) {
  var _arr = [];
  var _n = true;
  var _d = false;
  var _e = undefined;

  try {
    for (var _i = arr[Symbol.iterator](), _s; !(_n = (_s = _i.next()).done); _n = true) {
      _arr.push(_s.value);

      if (i && _arr.length === i) break;
    }
  } catch (err) {
    _d = true;
    _e = err;
  } finally {
    try {
      if (!_n && _i["return"] != null) _i["return"]();
    } finally {
      if (_d) throw _e;
    }
  }

  return _arr;
}

function _nonIterableRest() {
  throw new TypeError("Invalid attempt to destructure non-iterable instance");
}

function _slicedToArray(arr, i) {
  return _arrayWithHoles(arr) || _iterableToArrayLimit(arr, i) || _nonIterableRest();
}

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

function _objectSpread(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i] != null ? arguments[i] : {};
    var ownKeys = Object.keys(source);

    if (typeof Object.getOwnPropertySymbols === 'function') {
      ownKeys = ownKeys.concat(Object.getOwnPropertySymbols(source).filter(function (sym) {
        return Object.getOwnPropertyDescriptor(source, sym).enumerable;
      }));
    }

    ownKeys.forEach(function (key) {
      _defineProperty(target, key, source[key]);
    });
  }

  return target;
}

var regexpMap = {
  mobile: /^1\d{10}$/,
  // 手机校验
  url: /^((https|http|ftp|rtsp|mms)?:\/\/)[^\s]+/,
  // 网址校验
  identificationCard: /^\d{13,17}[0-9xX]$/,
  // 身份证号码格式校验
  tel: /(^(\d{3,4}-)?\d{7,8})$|(^1\d{10}$)/ // 校验电话号码

};

var Rules =
/*#__PURE__*/
function () {
  function Rules(regexpMap) {
    _classCallCheck(this, Rules);

    this.regexpMap = regexpMap;
    this.rules = {};
    this.createRegexpRule();
  }
  /**
   * 根据正则创建键值相同的校验方法
   */


  _createClass(Rules, [{
    key: "createRegexpRule",
    value: function createRegexpRule() {
      var _this = this;

      Object.keys(this.regexpMap).reduce(function (res, currentValue) {
        res[currentValue] = function (val) {
          return _this.regexpMap[currentValue].test(val + '');
        };

        return res;
      }, this.rules);
    }
    /**
     * 允许用户传入正则表达式来扩展
     */

  }, {
    key: "extendRegexp",
    value: function extendRegexp(regMap) {
      this.regexpMap = _objectSpread({}, this.regexpMap, regMap);
      this.createRegexpRule();
    }
    /**
     * 允许用户传入校验方法来扩展
     */

  }, {
    key: "extendValidator",
    value: function extendValidator(validatorMap) {
      this.rules = _objectSpread({}, this.rules, validatorMap);
    }
  }]);

  return Rules;
}();

var defaultRules = new Rules(regexpMap);

/* 创建偏函数，可提前接收部分参数 */
var partial = function partial(fn) {
  for (var _len = arguments.length, prevArgs = new Array(_len > 1 ? _len - 1 : 0), _key = 1; _key < _len; _key++) {
    prevArgs[_key - 1] = arguments[_key];
  }

  return function () {
    for (var _len2 = arguments.length, nextArgs = new Array(_len2), _key2 = 0; _key2 < _len2; _key2++) {
      nextArgs[_key2] = arguments[_key2];
    }

    return fn.apply(void 0, prevArgs.concat(nextArgs));
  };
};

var Validator =
/*#__PURE__*/
function () {
  function Validator(el, _ref, _ref2, _Vue) {
    var _this = this;

    var arg = _ref.arg,
        value = _ref.value,
        _ref$value = _ref.value,
        fields = _ref$value.fields,
        rules = _ref$value.rules,
        modifiers = _ref.modifiers;
    var context = _ref2.context;

    _classCallCheck(this, Validator);

    this.focusListener = function (_ref3) {
      var _ref3$target = _ref3.target,
          nodeName = _ref3$target.nodeName,
          name = _ref3$target.name;

      if (nodeName === 'INPUT' || nodeName === 'SELECT' || nodeName === 'TEXTAREA') {
        _this.$vec[name] && _this.vm.$set(_this.$vec, name, {
          pass: true,
          msg: ''
        });
      }
    };

    this.submitListener = function (e) {
      e.preventDefault();

      if (_this.autoCatch) {
        var _this$checkAll = _this.checkAll(),
            pass = _this$checkAll.pass;

        if (pass) {
          _this.submitMethod();
        }
      } else {
        _this.submitMethod();
      }
    };

    this.vm = context;
    this._Vue = _Vue;
    this.ref = context.$refs[value.ref];
    this.ref.validator = this.checkAll.bind(this); // 让 vue 组件本身可以校验所有数据

    this.formData = context[value.formData];
    this.fields = fields;
    this.rules = rules;
    this.autoCatch = !!modifiers.autoCatch;
    this.submitMethod = this.vm[arg];
    this.listeners = []; // 所有绑定事件的监听者

    this.createReactiveData();
    this.initEachEvent(el);
  }
  /* 初始化响应式对象 */


  _createClass(Validator, [{
    key: "createReactiveData",
    value: function createReactiveData() {
      var data = this.fields.reduce(function (res, key) {
        return _objectSpread({}, res, _defineProperty({}, key, {
          pass: true,
          msg: ''
        }));
      }, {});

      this._Vue.util.defineReactive(this.vm._data, '$vec', data);
    }
  }, {
    key: "createLengthValidate",

    /**
     * 将形如："min:5""max:8""min:5 max:8" 的字符串解析成校验字符串长度的校验函数
     * @param rule: string
     * @returns {function({length: number}): boolean}
     */
    value: function createLengthValidate(rule) {
      var reg = /^(m(ax|in):(\d+))(\sm(ax|in):(\d+)){0,1}$/;

      var _rule$match = rule.match(reg),
          _rule$match2 = _slicedToArray(_rule$match, 7),
          p2 = _rule$match2[2],
          p3 = _rule$match2[3],
          p4 = _rule$match2[4],
          p5 = _rule$match2[5],
          p6 = _rule$match2[6];

      var min, max;
      p2 === 'in' ? min = p3 : max = p3;

      if (p4 && p2 !== p5) {
        p5 === 'ax' ? max = p6 : min = p6;
      }

      if (min && max && ~~min > ~~max) throw '最小长度不能大于最大长度';
      return function (_ref4) {
        var length = _ref4.length;
        return !(min && ~~min > length || max && ~~max < length);
      };
    }
    /**
     * 验证 validator 的值类型，将其统一包装成函数
     * @param validator
     * @returns Function
     */

  }, {
    key: "createValidator",
    value: function createValidator(validator) {
      if (typeof validator === 'string') {
        if (defaultRules.rules[validator]) {
          return defaultRules.rules[validator];
        } else if (validator === 'required') {
          return function (val) {
            return !!val;
          };
        } else if (/^(m(ax|in):(\d+))(\sm(ax|in):(\d+)){0,1}$/.test(validator)) {
          return this.createLengthValidate(validator);
        } else {
          throw "\u60A8\u8FD8\u672A\u5B9A\u4E49 ".concat(validator, " \u8FD9\u6761\u89C4\u5219");
        }
      } else if (validator instanceof RegExp) {
        return function (val) {
          return validator.test(val);
        };
      } else if (typeof validator === 'function') {
        return validator;
      } else {
        throw 'validator 的值只能为函数或正则表达式';
      }
    }
  }, {
    key: "createValidateData",
    value: function createValidateData(res, name, target) {
      if (!res.pass && target.pass && res.validator !== target.validator) return;
      Object.assign(res, target);
      this.vm.$set(this.$vec, name, target);
    }
    /**
     * 检验某个字段的校验是否通过
     * @param val
     * @param rules
     * @param name
     * @returns {*}
     */

  }, {
    key: "verifySingle",
    value: function verifySingle(val, rules, name) {
      var res = {};
      var required = rules.some(function (rule) {
        return rule.validator === 'required';
      }); // 创建偏函数，接收部分参数

      var saveRes = partial(this.createValidateData.bind(this), res, name);

      for (var i = 0; i < rules.length; i++) {
        var _rules$i = rules[i],
            msg = _rules$i.msg,
            validator = _rules$i.validator;

        if (val === '' && !required) {
          saveRes({
            pass: true,
            msg: '',
            validator: validator
          });
          return res;
        } else if (!this.createValidator(validator)(val)) {
          saveRes({
            pass: false,
            msg: msg || '默认校验不通过消息',
            validator: validator
          });
          return res;
        }
      }

      saveRes({
        pass: true,
        msg: ''
      });
      return res;
    }
  }, {
    key: "verifyRule",
    value: function verifyRule(val, rule, name) {
      var res = this.$vec[name]; // 创建偏函数，接收部分参数

      var saveRes = partial(this.createValidateData.bind(this), res, name); // 第一个条件用来排除 值为空，并且非必填 的情况（该情况无需校验其他规则）

      if (!(val === '' && !this.ref[name].required) && !this.createValidator(rule.validator)(val)) {
        saveRes({
          pass: false,
          msg: rule.msg || '默认校验不通过消息',
          validator: rule.validator
        });
        return res;
      }

      saveRes({
        pass: true,
        msg: '',
        validator: rule.validator
      });
      return res;
    }
    /**
     * 提交的时候校验所有表单
     * @returns {{pass: boolean}}
     */

  }, {
    key: "checkAll",
    value: function checkAll() {
      var _this2 = this;

      var res = {
        pass: true
      };
      this.fields.forEach(function (item) {
        var checkOne = _this2.verifySingle(_this2.formData[item], _this2.rules[item], item);

        if (!checkOne.pass && res.pass) {
          res = _objectSpread({}, checkOne, {
            name: item
          });
        }
      });
      this.vm.$forceUpdate();
      return res;
    }
  }, {
    key: "bindEvent",
    value: function bindEvent(domName, rules) {
      var _this3 = this;

      var cloneRules = _toConsumableArray(rules); // 将是否必填的信息保存起来


      this.ref[domName].required = cloneRules.some(function (rule) {
        return rule.validator === 'required';
      });
      cloneRules.reverse().forEach(function (rule) {
        var listener = function listener(_ref5) {
          var target = _ref5.target;

          _this3.verifyRule(target.value, rule, domName);

          _this3.vm.$forceUpdate();
        };

        var trigger = rule.trigger || 'blur';

        _this3.ref[domName].addEventListener(trigger, listener); // 将解绑事件，添加到解绑列表中


        _this3.listeners.push(function () {
          _this3.ref[domName].removeEventListener(trigger, listener);
        });
      });
    }
  }, {
    key: "initEachEvent",
    value: function initEachEvent(el) {
      var _this4 = this;

      Object.keys(this.rules).forEach(function (item) {
        _this4.bindEvent(item, _this4.rules[item]);
      }); // 当鼠标聚焦时，这个表单元素需要正常

      this.ref.addEventListener('click', this.focusListener); // 绑定提交事件

      el.addEventListener('click', this.submitListener);
    }
  }, {
    key: "unbindEvent",
    value: function unbindEvent(el) {
      this.ref.removeEventListener('click', this.focusListener);
      el.removeEventListener('click', this.submitListener);
      this.listeners.forEach(function (fn) {
        return fn();
      });
    }
  }, {
    key: "$vec",
    get: function get() {
      return this.vm._data.$vec;
    }
  }]);

  return Validator;
}();
var rules = defaultRules;

var validate = (function (ValidatePlugin, _Vue) {
  return {
    inserted: function inserted(el, binding, vNode) {
      ValidatePlugin.validators instanceof Array || (ValidatePlugin.validators = []);
      ValidatePlugin.createValidator(ValidatePlugin, el, binding, vNode, _Vue);
      vNode.context.isFormComponent = true;
    },
    unbind: function unbind(el, binding, _ref) {
      var context = _ref.context;
      // 解绑事件
      ValidatePlugin.validators["cid_".concat(context._uid)].unbindEvent(el); // 通过组件的 id 来区分不同的表单

      ValidatePlugin.validators["cid_".concat(context._uid)] = null;
    }
  };
});

function isError (name) {
  return this._data.$vec ? !this._data.$vec[name].pass : false;
}

var deleteProp = function deleteProp(target, key) {
  var newObj = _objectSpread({}, target);

  newObj[key] && delete newObj[key];
  return newObj;
};

var deleteAllProp = function deleteAllProp(target) {
  var newObj = _objectSpread({}, target);

  Object.keys(newObj).forEach(function (key) {
    return newObj[key] = deleteProp(newObj[key], 'validator');
  });
  return newObj;
};

function verify (name) {
  var verifyData = _objectSpread({}, this._data.$vec);

  return name ? verifyData ? deleteProp(verifyData[name], 'validator') : {
    pass: true,
    msg: ''
  } : verifyData ? deleteAllProp(verifyData) : {};
}

var ValidatePlugin =
/*#__PURE__*/
function () {
  function ValidatePlugin() {
    _classCallCheck(this, ValidatePlugin);

    throw new Error('不允许实例化 ValidatePlugin');
  }

  _createClass(ValidatePlugin, null, [{
    key: "createValidator",
    value: function createValidator(ValidatePlugin, el, binding, vNode, _Vue) {
      ValidatePlugin.validators["cid_".concat(vNode.context._uid)] = new Validator(el, binding, vNode, _Vue);
    }
  }, {
    key: "install",
    value: function install(_Vue) {
      var options = arguments.length > 1 && arguments[1] !== undefined ? arguments[1] : {};

      if (!ValidatePlugin.validation) {
        ValidatePlugin.validation = {}; // 存放所有校验实例的容器

        options.regexpMap && rules.extendRegexp(options.regexpMap);
        options.validatorMap && rules.extendValidator(options.validatorMap);
      }

      _Vue.directive('validate', validate(ValidatePlugin, _Vue)); // 可供组件直接调用的校验方法


      _Vue.prototype.$isError = isError;
      _Vue.prototype.$verify = verify;
    }
  }]);

  return ValidatePlugin;
}();

exports.default = ValidatePlugin;
exports.rules = rules;
